/**
 * Zen for triadic handler. Plus proper http error handling and responder 
 */

/**
 * Default error handler
 */

var Kiai=require('3Kiai');

var errorHandler = function error(req, resp, err) {
	if (err) {
		resp.writeHead(500, {
			"Content-Type": "text/plain"
		});
		resp.end(err.stack + "\n");
		return;
	}
	resp.writeHead(404, {
		"Content-Type": "text/plain"
	});
	resp.end("Not Found\n");
};
/**
 * Default result handler
 */
var resultHandler = function result(req, resp, result) {
	resp.writeHead(200, {
		"Content-Type": "octet/stream"
	});
	resp.end(result);
};
/**
 * Zen uses a 'setup' pattern, returning a callable engine function
 */
module.exports= function (/*layers*/) {
	var error = function(a, b, err) {
		return engine.publish('error', a, b, err);
	};
	var layers=Array.prototype.concat([error],Array.prototype.slice.call(arguments).reverse());

	var L=layers.length-1;
	var first=layers[L];//first access optimization
	var nextSubscript = false;

	var nextHandler= function(a,b,err,res) {
			if (err) {
				return engine.publish('error', a, b, err); //err
			}
			return engine.publish('result', a, b, res);
	}
	
	// The real Z-Engine
	var engine= function (a,b) {
		var i=L;
		try {			
			var next= function(err,res) {
				if(!err&&!res) {
					if (nextSubscript) engine.publish('next',a,b,layers[i-1]);	//no subscription optimization				
					return layers[--i](a,b,next); 
				} 				
				return nextHandler(a,b,err,res);
			}
			return first(a,b,next);
		} catch (err) {
			return engine.publish('error', a, b, err);
		}
	}
	if (L==0){engine=first}; //no next
	/* EVENT EMITTER */
	Kiai.mixin(engine);
	engine.subscribe('error',errorHandler);
	engine.subscribe('result',resultHandler);
	engine.subscribe('newSubscription',function(type,listener){
		if (type === 'next' && L>0) nextSubscript=true; //no subscription & no next optimization
	});
	return engine;
};